@@ -8,6 +8,7 @@ import numpy as np
 import urllib, csv
 from urllib.request import urlopen
 import requests
+import statistics
 
 
 from survey_stats.functions import *
@@ -1106,9 +1107,9 @@ class TimeSeries(Data):
     def set_index(self, var: str | list, drop_var: bool = True) -> None:
         new_dict = {}
         if type(var)==str:
-            if var in self.values.keys():
+            if var in self.variables():
                 date_types = TimeSeries.type_of_dates(list(self.values[var].values()))
-                for v in self.values.keys():
+                for v in self.variables():
                     values = {}
                     if not (v == var and drop_var):
                         for i in self.values[var].keys():
@@ -1120,9 +1121,9 @@ class TimeSeries(Data):
             else:
                 raise ValueError(f"Error! {var} is not in variables of data.")
         elif type(var)==list:
-            if len(var) == len(self.index()):
+            if len(var) == len(self.dates):
                 date_types = TimeSeries.type_of_dates(var)
-                for v in self.values.keys():
+                for v in self.variables():
                     values = {}
                     for i, value in enumerate(self.values[v].values()):
                         date = date_to_standard(var[i], date_types)
@@ -1503,7 +1504,7 @@ class TimeSeries(Data):
         elif self.date_type == 'annual':
             pass    #TODO
 
-    def to_weekly(self, method:str='average')->TimeSeries:
+    def to_weekly(self, method:str='average', estimate_missings:bool=True)->TimeSeries:
         if self.date_type == 'daily':
             values = {}
             for var in self.variables():
@@ -1538,6 +1539,64 @@ class TimeSeries(Data):
                     values[var][endweek] = s/n if method == 'average' else s
             return TimeSeries('time', values)
 
+    def to_annual(self, method:str='average', estimate_missings:bool=True)->TimeSeries:
+        values, counts = {}, {}
+        for date in self.dates:
+            try:
+                year = int(date[:4])
+                for var in self.variables():
+                    if not var in values.keys():
+                        values[var] = {}
+                    if year in values[var].keys():
+                        values[var][year] += float(self.values[var][date])
+                    else:
+                        values[var][year] = float(self.values[var][date])
+                    if year in counts.keys():
+                        counts[year] += 1
+                    else:
+                        counts[year] = 1
+            except:
+                raise ValueError(f'Error! {date} is not standard: yyyy-q or yyyy-mm or yyyy-mm-dd.')
+        frequency = statistics.mode(counts.values())
+        if method == 'average':
+            for year in counts.keys():
+                if counts[year] == frequency:
+                    for var in values.keys():
+                        values[var][year] /= frequency
+                elif estimate_missings:
+                    if year-1 in counts.keys():
+                        for var in values.keys():
+                            r_1 = sum([self.values[var][date.replace(str(year), str(year-1))] 
+                                            for date in self.dates
+                                                if date[:4]==str(year) and 
+                                                not is_nan(self.values[var][date.replace(str(year), str(year-1))], True)])
+                            values[var][year] = (values[var][year-1]/r_1 * values[var][year])/frequency
+                    else:
+                        for var in values.keys():
+                            values[var][year] *= 1/counts[year]
+                else:
+                    for var in values.keys():
+                        values[var][year] = np.nan
+        else:       #sum
+            for year in counts.keys():
+                if counts[year] != frequency:
+                    if estimate_missings:
+                        if year-1 in counts.keys():
+                            for var in values.keys():
+                                r_1 = sum([float(self.values[var][date.replace(str(year), str(year-1))])
+                                            for date in self.dates
+                                                if date[:4]==str(year) and 
+                                                not is_nan(self.values[var][date.replace(str(year), str(year-1))])])
+                                values[var][year] = values[var][year-1]/r_1 * values[var][year]
+                        else:
+                            for var in values.keys():
+                                values[var][year] *= frequency/counts[year]
+                    else:
+                        for var in values.keys():
+                            values[var][year] = np.nan
+
+        return TimeSeries(values=values)
+
     @classmethod
     def read_csv(cls, path_file:str, data_type:str='cross', na:any='', index:str='index')->TimeSeries:
         data = Data.read_csv(path_file, data_type, na, index)
